/*
 *   This file is automatically generated by Plasma UI Compiler
 *   DO NOT CHANGE THE CONTENTS, IT IS AUTOMATICALLY REGENERATED
 */

#ifndef ${HEADER_ID}
#define ${HEADER_ID}

/* Includes */
${INCLUDES}

/* Defines */
${DEFINES}

namespace Ui {

class ${CLASS_NAME}
{

protected:

    ${DECLARATION}
    
    void setupUi(${PARENT_OBJECT_TYPE} * root)
    {
        /* Initialization */
        ${INITIALIZATION}
        
        /* Setup */
        ${SETUP}
    }
};

} // namespace Ui

#endif /* ${HEADER_ID} */
